
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "Hardware.h"
#include "Globals.h"
#include "port_driver.h"

#include "Audio.h"
#include "TC_driver.h"

#ifndef __ASSEMBLER__
#define __ASSEMBLER__
#else
#endif
//I use the assembler definitions

//oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
//+-----------------------------------------------------------------------
//	Function name : Audio_Init
//	Returns :		none
//	Parameters :	audioNumber (is the number of the audio in dataflash)
//	Purpose :		Play an audio source
//------------------------------------------------------------------------
//oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
void  Audio_Play(void)
{   uint16_t k_sample = SAMPLE_6000;

	uint8_t Pwm_data;
	uint16_t V_meeter_time = 0;
	uint32_t index = 1;



  //ooooooooooooooooooooooooooooo
  //o SET DAC PHARAMETER
  //ooooooooooooooooooooooooooooo
    //dual channel conversion
	DACB_CTRLB = DAC_CHSEL_DUAL_gc; //0;
	//left adjusted and Vcc ref
    DACB_CTRLC = DAC_LEFTADJ_bm | DAC_REFSEL_AVCC_gc;
   // 2uS of conversion interval at 32000000Mhz
	DACB_TIMCTRL = DAC_CONINTVAL1_bm | DAC_CONINTVAL2_bm |DAC_REFRESH3_bm |DAC_REFRESH2_bm | DAC_REFRESH1_bm | DAC_REFRESH0_bm;
    DACB_CTRLA   = DAC_CH0EN_bm | DAC_CH1EN_bm |DAC_ENABLE_bm;

 

  //ooooooooooooooooooooooooooooo
  //o ENABLE TIMER 0 TO COUNT
  //O NOTE: IF YOU WANT TO USE TIMER
  //O SETTINGS INTO OTHER PLACES,
  //O USE THE ANSI C FORM SHOWED ON
  //O ATMEL DEMO
  //ooooooooooooooooooooooooooooo
	// Set period/TOP value.
    TC_SetPeriod( &TCC0, SAMPLE_8000);
    //set the counter 
    TC_SetCount( &TCC0, 0x00 );
	// Select clock source. 
	TC0_ConfigClockSource( &TCC0, TC_CLKSEL_DIV256_gc );
	// Enable overflow interrupt at low level 
	TC0_SetOverflowIntLevel( &TCC0, TC_OVFINTLVL_LO_gc );

	Play_in_progress = 1; //if = 1 I  can't access to dataflash for other things	

    
    Pwm_data =  pgm_read_byte(&Music[0]); //first byte is the sample rate

	switch (Pwm_data)
	{
	   case K_SAMPLE_6000:
            k_sample = SAMPLE_6000;
       break;
	   case K_SAMPLE_8000:
            k_sample = SAMPLE_8000;
       break;
	   case K_SAMPLE_16000:
            k_sample = SAMPLE_16000;
       break;
	   case K_SAMPLE_32000:
            k_sample = SAMPLE_32000;
       break;
	   default:
            k_sample = SAMPLE_8000;
       break;
    }

    TC_SetPeriod( &TCC0, k_sample);


   while (Pwm_data != 0)
   {
 

        while( !Flag_Audio);               //use the bit1 of ADCSR for say ok  wait for timer1 overflow interrupt            
	    	_delay_us(5);
        
            Pwm_data =  pgm_read_byte(&Music[index++]); 

	        DACB_CH0DATA = Pwm_data<<8;       //for 8 bit conversion use only the high byte
	        DACB_CH1DATA = Pwm_data<<8; //???
	        if (V_meeter_time++ >= 100)
	        {  
	           if(Pwm_data <= 128) 
	     	       PORTE.OUT = 0xFF;
	           else if((Pwm_data >= 129) &&(Pwm_data <= 140))
	     		   PORTE.OUT = 0xEE;
	           else if((Pwm_data >= 141) &&(Pwm_data <= 155))
	     		   PORTE.OUT = 0xCC;
	           else if((Pwm_data >= 156) &&(Pwm_data <= 170))
	     		   PORTE.OUT = 0x88;
	           else if(Pwm_data >= 171)
	     		   PORTE.OUT = 0x00;
	           else;


			   V_meeter_time = 0;
	         }
	         else;
	     

        CLR_AUDIO_FLAG; 			      // clear thee audio bit
     }

 
       DACB_CH0DATA = 127;       //for 8 bit conversion use only the high byte
       DACB_CH1DATA = 127; //???

 	TCC0_CTRLA = TC_CLKSEL_OFF_gc ; //stop timer 0
	TCC0_INTCTRLA = 0;

					


	Play_in_progress = 0; //if = 0 I  can access to dataflash for other things
}



//oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
//+-----------------------------------------------------------------------
//	Function name : Timer1 overflow 
//	Returns :		none
//	Parameters :	none
//	Purpose :		Interrupt tier 1 at 8Mhz/1 an audio source
//------------------------------------------------------------------------
//oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

ISR(TCC0_OVF_vect)
{
   SET_AUDIO_FLAG;
 //  Print_Char(90);
}


/*
ISR(TIMER1_OVF_vect)
{
    ACSR |= T1_OVF; 
}

*/

/*! \brief Interrupt service routine for Example4()
 *
 *  When initialized using Example4(), this ISR is called on every rising edge
 *  on PC0.
 */
ISR(PORTF_INT0_vect)
{

     flagbutton = 1;

}



